#load data
load("~/data.rda")

#install.packages("metafor",dep=T)
library(metafor)

#find the best model####
#starting model
res <- rma.mv(SMCR ~ t2 + t3 + t4 - 1, var.SMCR, random = ~ 1 | study/sample/outcome/comp, data=dat)
res

#unstructured variance-covariance matrix
res.UN <- rma.mv(SMCR ~ t2 + t3 + t4 - 1, var.SMCR, random = ~ 1 | study/sample/outcome/comp, struct = "UN", data=dat)
res.UN

#autocorrelated random effects
res.AR <- rma.mv(SMCR ~ t2 + t3 + t4 - 1, var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="AR", data=dat)
res.AR

#allow for different amounts of heterogeneity for the three contrasts:
res.HAR <- rma.mv(SMCR ~ t2 + t3 + t4 - 1, var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"), data=dat)
res.HAR

#compare models using likelihood ratio tests:
anova(res, res.UN)
anova(res, res.AR)
anova(res, res.HAR)
anova(res.AR, res.UN)
anova(res.HAR, res.UN)
anova(res.AR, res.HAR)
#->use HAR for all models


#cluster-robust estimates of SEs according to Hedges et al. (2010)####
res.robust <- robust(res.HAR, cluster=dat$study) #use studies to cluster
summary(res.robust)

#H1a####
#Hypothesis 1: RTs decrease with the number of test administrations, with the largest RT reduction from first to second test administration
res.robust

#1.2
predict(res.robust, newmods = c(+1, 0, 0), tau2.levels = "1vs2")
anova(res.robust, L = c(+1, 0, 0))

#2.3
predict(res.robust, newmods = c(-1,+1, 0), tau2.levels = "1vs3") 
anova(res.robust, L = c(-1,+1, 0))

#3.4
predict(res.robust, newmods = c( 0,-1,+1), tau2.levels = "1vs4")
anova(res.robust, L = c( 0,-1,+1)) 

#1.2 vs 2.3
anova(res.robust, L = c(+1, 0, 0)  - c(-1,+1, 0)) 
predict(res.robust, newmods = c(+1, 0, 0)  - c(-1,+1, 0), tau2.levels = "1vs3")

#2.3 vs 3.4
anova(res.robust, L = c(-1,+1, 0) - c( 0,-1,+1)) 
predict(res.robust, newmods = c(-1,+1, 0) - c( 0,-1,+1), tau2.levels = "1vs3")

#tau
#overall tau
#1.2
tau1.2.sqrt <- (res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[1])
tau1.2 <- sqrt(res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[1])
tau1.2
#1.3
tau1.3.sqrt <- (res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[2])
tau1.3 <- sqrt(res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[2])
tau1.3
#1.4
tau1.4.sqrt <- (res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[3])
tau1.4 <- sqrt(res.HAR$sigma2[1]+res.HAR$sigma2[2]+res.HAR$sigma2[3]+res.HAR$tau2[3])
tau1.4

#without Lyall
dat.Ly <- subset(dat, study != 31)

res.HAR.Ly <- rma.mv(SMCR ~ t2 + t3 + t4 - 1, var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"), data=dat.Ly)
res.HAR.Ly
#cluster-robust estimates of SEs according to Hedges et al. (2010)
res.robust.Ly <- robust(res.HAR.Ly, cluster=dat.Ly$study) #use studies to cluster
summary(res.robust.Ly)

#H1a
#Hypothesis 1: RTs decrease with the number of test administrations, with the largest RT reduction from first to second test administration
res.robust.Ly

#1.2
predict(res.robust.Ly, newmods = c(+1, 0, 0), tau2.levels = "1vs2")
anova(res.robust.Ly, L = c(+1, 0, 0))

#2.3
predict(res.robust.Ly, newmods = c(-1,+1, 0), tau2.levels = "1vs3") 
anova(res.robust.Ly, L = c(-1,+1, 0))

#3.4
predict(res.robust.Ly, newmods = c( 0,-1,+1), tau2.levels = "1vs4")
anova(res.robust.Ly, L = c( 0,-1,+1)) 

#1.2 vs 2.3
anova(res.robust.Ly, L = c(+1, 0, 0)  - c(-1,+1, 0)) 
predict(res.robust.Ly, newmods = c(+1, 0, 0)  - c(-1,+1, 0), tau2.levels = "1vs3")

#2.3 vs 3.4
anova(res.robust.Ly, L = c(-1,+1, 0) - c( 0,-1,+1)) 
predict(res.robust.Ly, newmods = c(-1,+1, 0) - c( 0,-1,+1), tau2.levels = "1vs3")

#tau
#overall tau
#1.2
tau1.2.sqrt.Ly <- (res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[1])
tau1.2.Ly <- sqrt(res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[1])
tau1.2.Ly
#1.3
tau1.3.sqrt.Ly <- (res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[2])
tau1.3.Ly <- sqrt(res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[2])
tau1.3.Ly
#1.4
tau1.4.sqrt.Ly <- (res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[3])
tau1.4.Ly <- sqrt(res.HAR.Ly$sigma2[1]+res.HAR.Ly$sigma2[2]+res.HAR.Ly$sigma2[3]+res.HAR.Ly$tau2[3])
tau1.4.Ly


#TMT and Stroop####
res.test.HAR <- rma.mv(SMCR ~ 
                         t2:factor(test.cat) + t3:factor(test.cat) + t4:factor(test.cat) - 1, 
                       var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", data=dat)
res.test.HAR

#robust variance estimation
res.test.robust <- robust(res.test.HAR, cluster=dat$study)
res.test.robust

#Stroop
#2.3
anova(res.test.robust, L = c( 0,-1,0, 0,+1,0, 0,0))
predict(res.test.robust, newmods = c( 0,-1,0, 0,+1,0, 0,0), tau2.levels = "1vs3")
#1.2vs2.3
anova(res.operation.robust, L = c( 0,+1,0, 0,0,0, 0,0) - c( 0,-1,0, 0,+1,0, 0,0))
predict(res.operation.robust, newmods = c( 0,+1,0, 0,0,0, 0,0) - c( 0,-1,0, 0,+1,0, 0,0), tau2.levels = "1vs3")

#TMT
#2.3
anova(res.test.robust, L = c( 0,0,-1, 0,0,+1, 0,0))
predict(res.test.robust, newmods = c( 0,0,-1, 0,0,+1, 0,0), tau2.levels = "1vs3")
#1.2vs2.3
anova(res.operation.robust, L = c( 0,0,+1, 0,0,0, 0,0) - c( 0,0,-1, 0,0,+1, 0,0)) 
predict(res.operation.robust, newmods = c( 0,0,+1, 0,0,0, 0,0) - c( 0,0,-1, 0,0,+1, 0,0), tau2.levels = "1vs3")

#Do Stroop and TMT effects differ?
anova(res.test.robust, L = c( 0,-1,+1, 0,0,0, 0,0))
anova(res.test.robust, L = c(0,0,0, 0,-1,+1,0,0)) 

#task complexity####
table(data.1vs2$cmplx)
table(data.1vs3$cmplx)
table(data.1vs4$cmplx)#less than 10 simple task outcomes -> no subgroup analysis

res.cmplx.HAR <- rma.mv(SMCR ~ t2:factor(cmplx) + t3:factor(cmplx)  + t4:factor(cmplx) -1, var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"),
                        data=dat)
res.cmplx.HAR
#robust variance estimation
res.cmplx.robust <- robust(res.cmplx.HAR, cluster=dat$study)
summary(res.cmplx.robust)

anova(res.cmplx.robust, L = c(-1,+1, 0,0, 0,0))
anova(res.cmplx.robust, L = c( 0,0, -1,+1,0,0))

#1.2 vs 2.3
#simple
anova(res.cmplx.robust, L = c(-1,0, +1,0, 0,0)) 
predict(res.cmplx.robust, newmods = c(-1,0, +1,0, 0,0), tau2.levels = "1vs3")
#complex
anova(res.cmplx.robust, L = c(0,-1, 0,+1, 0,0)) 
predict(res.cmplx.robust, newmods = c(0,-1, 0,+1, 0,0), tau2.levels = "1vs3")
#simplex vs complex
anova(res.cmplx.robust, L = c(0,-1, 0,+1, 0,0)-c(-1,0, +1,0, 0,0)) 
predict(res.cmplx.robust, newmods = c(0,-1, 0,+1, 0,0)-c(-1,0, +1,0, 0,0), tau2.levels = "1vs3")


#tau
#overall tau
#1.2
tau1.2.sqrt.cmplx <- (res.cmplx.HAR$sigma2[1]+res.cmplx.HAR$sigma2[2]+res.cmplx.HAR$sigma2[3]+res.cmplx.HAR$tau2[1])
tau1.2.cmplx <- sqrt(res.cmplx.HAR$sigma2[1]+res.cmplx.HAR$sigma2[2]+res.cmplx.HAR$sigma2[3]+res.cmplx.HAR$tau2[1])
tau1.2.cmplx
#1.3
tau1.3.sqrt.cmplx <- (res.cmplx.HAR$sigma2[1]+res.cmplx.HAR$sigma2[2]+res.cmplx.HAR$sigma2[3]+res.cmplx.HAR$tau2[2])
tau1.3.cmplx <- sqrt(res.cmplx.HAR$sigma2[1]+res.cmplx.HAR$sigma2[2]+res.cmplx.HAR$sigma2[3]+res.cmplx.HAR$tau2[2])
tau1.3.cmplx

#incremental tau
#1.2
(tau1.2.sqrt - tau1.2.sqrt.cmplx)/tau1.2.sqrt
#1.3
(tau1.3.sqrt - tau1.3.sqrt.cmplx)/tau1.3.sqrt

#TR interval####
res.week.HAR <- rma.mv(SMCR ~ t2 + t3 + t4
                       + t2:week + t3:week + t4:week - 1, 
                       var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", data=dat)
res.week.HAR
#robust variance estimation
res.week.robust <- robust(res.week.HAR, cluster=dat$study)
summary(res.week.robust)

#Hypothesis 4: Retest effects will decline with a longer time interval between administrations.
anova(res.week.robust, L = c(0,0,0, +1, 0, 0)) ### effect of mod on 1vs2
anova(res.week.robust, L = c(0,0,0, 0,+1, 0)) ### effect of mod on 1vs3
anova(res.week.robust, L = c(0,0,0, 0, 0, +1)) ### effect of mod on 1vs4

#t23
anova(res.week.robust, L = c(0,+1,0, 0,0, 0)-c(+1,0,0, 0,0,0)) ### effect of mod on 2vs3: intercept
predict(res.week.robust, newmods = c(0,+1,0, 0,0, 0)-c(+1,0,0, 0,0,0), tau2.levels = "1vs3") ### intercept
anova(res.week.robust, L = c(0,0,0, 0,+1, 0)-c(0,0,0, +1,0,0)) ### effect of mod on 2vs3: weight
predict(res.week.robust, newmods = c(0,0,0, 0,+1, 0)-c(0,0,0, +1,0,0), tau2.levels = "1vs3") ### weight

#t34
anova(res.week.robust, L = c(0,0,+1, 0,0, 0)-c(0,+1,0, 0,0,0)) ### effect of mod on 3vs4
predict(res.week.robust, newmods = c(0,0,+1, 0,0, 0)-c(0,+1,0, 0,0,0), tau2.levels = "1vs4") 
anova(res.week.robust, L = c(0,0,0,  0,0, +1) - c(0,0,0, 0,+1,0)) ### effect of mod on 3vs4
predict(res.week.robust, newmods = c(0,0,0,  0,0, +1) - c(0,0,0, 0,+1,0), tau2.levels = "1vs4") 

#tau
#overall tau
#1.2
tau1.2.sqrt.week <- (res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[1])
tau1.2.week <- sqrt(res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[1])
tau1.2.week
#1.3
tau1.3.sqrt.week <- (res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[2])
tau1.3.week <- sqrt(res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[2])
tau1.3.week
#1.4
tau1.4.sqrt.week <- (res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[3])
tau1.4.week <- sqrt(res.week.HAR$sigma2[1]+res.week.HAR$sigma2[2]+res.week.HAR$sigma2[3]+res.week.HAR$tau2[3])
tau1.4.week

#incremental tau
#1.2
(tau1.2.sqrt - tau1.2.sqrt.week)/tau1.2.sqrt
#1.3
(tau1.3.sqrt - tau1.3.sqrt.week)/tau1.3.sqrt
#1.4
(tau1.4.sqrt - tau1.4.sqrt.week)/tau1.4.sqrt

#including week and complexity
res.week.cmplx.HAR <- rma.mv(SMCR ~ t2 + t3 + t4
                            + t2:factor(cmplx):week + t3:factor(cmplx):week + t4:factor(cmplx):week -1, 
                            var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"), data=dat)
res.week.cmplx.HAR
#robust variance estimation
res.week.cmplx.robust <- robust(res.week.cmplx.HAR, cluster=dat$study)
res.week.cmplx.robust

#age####
res.age.HAR <- rma.mv(SMCR ~ t2 + t3 + t4
                        + t2:age.M + t3:age.M + t4:age.M - 1, 
                        var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"), data=dat)
res.age.HAR
#robust variance estimation
res.age.robust <- robust(res.age.HAR, cluster=dat$study)
res.age.robust

#Hypothesis 5: Retest effects will decline with age.
anova(res.age.robust, L = c(0,0,0, +1, 0, 0)) ### effect of mod on 1vs2
anova(res.age.robust, L = c(0,0,0, 0,+1, 0)) ### effect of mod on 1vs3
anova(res.age.robust, L = c(0,0,0, 0, 0, +1)) ### effect of mod on 1vs4

#t23
anova(res.age.robust, L = c(-1,+1,0, 0,0, 0)) ### effect of mod on 2vs3: intercept
predict(res.age.robust, newmods = c(-1,+1,0, 0,0, 0), tau2.levels = "1vs3") ###intercept

anova(res.age.robust, L = c(0,0,0, -1,+1, 0)) ### effect of mod on 2vs3: weight
predict(res.age.robust, newmods = c(0,0,0, -1,+1, 0), tau2.levels = "1vs3") ###weight

#t34
anova(res.age.robust, L = c(0,-1,+1, 0,0,0)) ### effect of mod on 3vs4: intercept
predict(res.age.robust, newmods = c(0,-1,+1, 0,0, 0), tau2.levels = "1vs3") ###intercept

anova(res.age.robust, L = c(0,0,0,  0,-1,+1)) ### effect of mod on 3vs4: weight
predict(res.age.robust, newmods = c(0,0,0,  0,-1,+1), tau2.levels = "1vs3") ###weight

anova(res.age.robust, L = c(0,0,0, -1,+1, 0) - c(0,0,0, +1, 0, 0)) ### test whether effect of mod on 2vs3 is different from 1vs2
anova(res.age.robust, L = c(0,0,0, -1,+1, 0) - c(0,0,0, 0, -1, +1)) ### test whether effect of mod on 2vs3 is different from 1vs2

#tau
#overall tau
#1.2
tau1.2.sqrt.age <- (res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[1])
tau1.2.age <- sqrt(res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[1])
tau1.2.age
#1.3
tau1.3.sqrt.age <- (res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[2])
tau1.3.age <- sqrt(res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[2])
tau1.3.age
#1.4
tau1.4.sqrt.age <- (res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[3])
tau1.4.age <- sqrt(res.age.HAR$sigma2[1]+res.age.HAR$sigma2[2]+res.age.HAR$sigma2[3]+res.age.HAR$tau2[3])
tau1.4.age

#incremental tau
#1.2
(tau1.2.sqrt - tau1.2.sqrt.age)/tau1.2.sqrt
#1.3
(tau1.3.sqrt - tau1.3.sqrt.age)/tau1.3.sqrt
#1.4
(tau1.4.sqrt - tau1.4.sqrt.age)/tau1.4.sqrt

#including age and complexity
res.age.cmplx.HAR <- rma.mv(SMCR ~ t2 + t3 + t4
                      + t2:factor(cmplx):age.M + t3:factor(cmplx):age.M + t4:factor(cmplx):age.M -1, 
                      var.SMCR, random = list(~ 1 | study/sample/outcome, ~ comp | interaction(study, sample, outcome)), struct="HAR", control=list(optimizer="optim"), data=dat)
res.age.cmplx.HAR
#robust variance estimation
res.age.cmplx.robust <- robust(res.age.cmplx.HAR, cluster=dat$study)
res.age.cmplx.robust


#associations between moderators####
cor.test(dat$time, dat$age.M)
cor.test(dat$cmplx, dat$age.M)
cor.test(dat$time, dat$cmplx)

#funnel plot####
funnel(res.HAR)

#forest plot####
pdf(file=paste("Forest.pdf",sep=""),width=10,height=44)
par(mar=c(5,4,.1,2))
forest(res.robust,
       slab = paste(dat$authors.year,",", dat$comp),
       efac = .1, ylim = c(218,7),
       mlab = "Overall Effect Size")
title("Forest Plot",
      line = -6)
dev.off()
